;nyquist plug-in
;version 4
;type tool
;debugflags trace
;name "Label Summary"
;author "Steve Daulton"
;release 3.1.3.1
;copyright "GNU General Public License v2.0 or later"

;; By Steve Daulton, www.audionyq.com 2022.
;;
;; Released under terms of the GNU General Public License v2.0 or later:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html


(defun s-to-hms (ss &optional (short 1))
  ;;; Convert seconds into hh:mm:ss
  (let* ((hh (truncate (/ ss 3600.0)))
         (mm (- (truncate (/ ss 60.0))(* hh 60)))
         (dec (- ss (truncate ss)))
         (ss (rem (truncate ss) 60)))
      (setf ss (+ ss dec))
      (setf *float-format* (format nil "%#.3f"))
      (flet ((pad (num) (cond ((> num 9) num)
                              ((> num 0) (format nil "0~a" num))
                              (T "00.000"))))
        (format nil "~a~a~a~a~a"
                    (if (> hh 0) hh "")
                    (if (> hh 0) ":" "")
                    (if (> mm 0)
                        (if (> hh 0) (pad mm) mm)
                        "")
                    (if (> mm 0) ":" "")
                    (if (> (+ hh mm) 0) (pad ss) ss)))))


(defun get-track-name (data id)
  ;;; Return name of track from track data and track id.
  (let ((trk (nth id data)))
    (second (assoc 'name trk))))


(defmacro get-track-stats (data points regions initial final tbefore)
  ;;; Get number of point and region labels
  ;;; 'data' is in the form '((t0 t1 txt1)(t2 t3 txt2)...)
  `(dolist (label ,data)
      ;; 'initial' is passed as NIL. Only set it once.
      ;; 'tbefore' is a counter for all label tracks.
      ;; Note that label times may be integers (bug? Jan 2022)
      (unless ,initial
        (setf ,initial (float (first label))))
      (when (< (first label) 0)
        (incf tbefore))
      ;; We only need the last value, but no point testing.
      (setf ,final (float (second label)))
      (if (= (second label) (first label))
          (incf ,points)
          (incf ,regions))))


(defmacro tally (var &rest vals)
  ;;; Add vals to var
  `(setf ,var (+ ,var ,@vals)))


(defun printout (tpoints tregions tbefore tinitial tfinal summary)
  ;;; Print stats to debug window.
  (setf *float-format* "%.3f")
  (setf output
      (format nil "Total: ~a labels: ~
                  ~a point labels, ~a region labels.~%"
              (+ tpoints tregions) tpoints tregions))
  (string-append output
      (format nil "First label at: ~a~a~a~a~%~
                  Final label at: ~a~%~%"
              (s-to-hms tinitial)
              (if (> tbefore 0) " [" "")
              (if (> tbefore 0) tbefore "")
              (if (< tinitial 0) " Before zero!]" "")
              (s-to-hms tfinal))
      "Details: (Name, Start Time to End Time)\n\n")
  (dolist (trk summary output)  ;return output
    (string-append output
        (format nil "Track ~a:  (~s  ~a to ~a)~%"
                (nth 0 trk)   ;trknum
                (nth 1 trk)   ;tname
                (s-to-hms (nth 4 trk))  ;initial
                (s-to-hms (nth 5 trk))) ;final
        (format nil "~a point labels, ~a region labels.~%~%"
            (nth 2 trk)
            (nth 3 trk)))))


(defun do-summary ()
  (let ((labels (aud-get-info "Labels"))
        (tracks (aud-get-info "Tracks"))
        (summary ())
        (tpoints 0)   ;total number of point labels
        (tregions 0)  ;total number of region labels
        (tbefore 0)   ;total that start before t=0
        tinitial      ;earliest label
        tfinal        ;last label
        ;; temp vars for per track data
        (points 0)
        (regions 0)
        trknum tname initial final)
    (unless labels (throw 'err "No label tracks in project."))
    (dolist (ltrk labels)  ;for each label track...
      ;; Gather label track data.
      (setf trknum (first ltrk))  ;track id
      (setf tname (get-track-name tracks trknum))
      (get-track-stats (second ltrk) points regions initial final tbefore)
      (push (list (1+ trknum) tname points regions initial final) summary)
      ;; Update totals.
      (tally tpoints points)
      (tally tregions regions)
      (if tinitial
          (setq tinitial (min tinitial initial)
                tfinal (max tfinal final))
          (setq tinitial initial
                tfinal final))
      ;; Reset for next label track
      (psetq points 0 regions 0
             initial nil final nil))
    (unless tinitial (throw 'err "No labels in project."))
    (printout tpoints tregions tbefore tinitial tfinal (reverse summary))))


(catch 'err (do-summary))
